<?php

namespace Drupal\module_builder\EntityHandler;

use Drupal\Core\Entity\EntityHandlerInterface;
use Drupal\Core\Entity\EntityTypeInterface;

/**
 * Reads data from the code_builder annotation on a component entity type.
 */
class ComponentSectionFormHandler {

  /**
   * Constructs a new ComponentSectionFormHandler object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   */
  public function __construct(EntityTypeInterface $entity_type) {
    $this->entityTypeId = $entity_type->id();
    $this->entityType = $entity_type;
    $this->codeBuilderAnnotation = $this->entityType->get('code_builder');
  }

  /**
   * Gets the form operations defined by the component sections.
   *
   * @return string[]
   *   An array of form operation names.
   */
  public function getFormOperations() {
    $form_properties = $this->codeBuilderAnnotation['section_forms'];

    if (!isset($form_properties['name'])) {
      throw new \Exception("Entity type {$entity_type_id} defines the code_builder annotation, but doesn't have a name form set.");
    }

    // The 'name' form gets special treatment.
    // TODO: currently hardcoded in the class.
    // TODO: rename this class to a more generic name.
    //$entity_type->setFormClass('name', \Drupal\module_builder\Form\ModuleNameForm::class);
    unset($form_properties['name']);

    $form_operations = array_keys($form_properties);

    // The 'misc' form always gets added: all other properties go in that.
    $form_operations[] = 'misc';

    return $form_operations;
  }

  /**
   * Gets the route data for the section forms.
   *
   * @return array
   *   An array whose keys are the path components and values are route titles.
   */
  public function getFormTabRoutePaths() {
    // TODO Do this prep in construct!
    $section_form_data = $this->codeBuilderAnnotation['section_forms'];

    // The 'name' form gets special treatment.
    unset($section_form_data['name']);

    $route_data = [];

    foreach ($section_form_data as $form_op => $section) {
      $route_data[$form_op] = $section['title'];
    }

    // Add the 'misc' section.
    $route_data['misc'] = "Edit miscellaneous components";

    return $route_data;
  }

  /**
   * Gets the local tasks data for the section forms.
   *
   * @return array
   *   An array whose keys are the path components and values are task titles.
   */
  public function getFormTabLocalTasksData() {
    // TODO Do this prep in construct!
    $section_form_data = $this->codeBuilderAnnotation['section_forms'];

    // The 'name' form gets special treatment.
    unset($section_form_data['name']);

    $route_data = [];

    foreach ($section_form_data as $form_op => $section) {
      $route_data[$form_op] = $section['tab_title'];
    }

    // Add the 'misc' section.
    $route_data['misc'] = "Misc";

    return $route_data;
  }

  /**
   * Gets the component properties to show for a section form.
   *
   * @param string $form_op
   *   The form operation name.
   *
   * @return string[]
   *   An array of property names for the component.
   */
  public function getSectionFormComponentProperties($form_op) {
    $form_properties = $this->codeBuilderAnnotation['section_forms'];

    return $form_properties[$form_op]['properties'];
  }

  /**
   * Gets all the component properties set to show in section forms.
   *
   * @return string[]
   *   An array of property names for the component.
   */
  public function getUsedComponentProperties() {
    $form_properties = $this->codeBuilderAnnotation['section_forms'];

    $used_component_properties = [];
    foreach ($form_properties as $form_op => $section_data) {
      $used_component_properties = array_merge($used_component_properties, $section_data['properties']);
    }

    return $used_component_properties;
  }

}
